@@ -72,6 +72,11 @@ CONFIG_SCHEMA = {
     "properties": {
         "mirror": {"$ref": "#/definitions/mirrorList"},
         "gitlab_merge": {"$ref": "#/definitions/gitlabMerge"},
+        "gitlab_promote": {
+            "type": "array",
+            # TODO: actually it's not 100% identical to gitlabMerge.
+            "items": {"$ref": "#/definitions/gitlabMerge"},
+        },
         "git_config": {"type": "object"},
         "commitmsg": {
             "type": "string",
@@ -91,8 +96,7 @@ class GitlabMergeComments:
 
 
 @dataclass
-class GitlabMerge:
-    enabled: bool = False
+class GitlabCommon:
     src: str = "latest"
     dest: str = ""
     title: str = "Update mirror"
@@ -132,10 +136,6 @@ class GitlabMerge:
     def push_url_final(self) -> str:
         return self._token_auth(self.push_url)
 
-    @property
-    def api_v4_url_final(self) -> str:
-        return self._token_auth(self.api_v4_url)
-
     def _token_auth(self, url: str):
         if not url.startswith("https://"):
             # Not using token-based auth
@@ -148,6 +148,21 @@ class GitlabMerge:
         return "".join(["https://", "token:", self.token_final, "@", url_noscheme])
 
 
+@dataclass
+class GitlabMerge(GitlabCommon):
+    enabled: bool = False
+
+
+@dataclass
+class GitlabPromote(GitlabCommon):
+    title: str = "Promote changes"
+    description: str = "Automated promotion between branches."
+
+    @property
+    def working_branch(self) -> str:
+        return f"mirror-tool/promote-{self.src}-to-{self.dest}"
+
+
 @dataclass
 class Mirror:
     url: str
@@ -166,6 +181,17 @@ class Config:
         raw["comment"] = GitlabMergeComments(**raw_comment)
         return GitlabMerge(**raw)
 
+    @property
+    def gitlab_promote(self) -> List[GitlabPromote]:
+        out = []
+        raw = self._raw.get("gitlab_promote") or []
+        for elem in raw:
+            elem = elem.copy()
+            raw_comment = elem.get("comment") or {}
+            elem["comment"] = GitlabMergeComments(**raw_comment)
+            out.append(GitlabPromote(**elem))
+        return out
+
     @property
     def git_config(self) -> Dict[str, Any]:
         return self._raw.get("git_config") or {}
@@ -187,6 +213,14 @@ class Config:
     def validate(self) -> None:
         jsonschema.validate(self._raw, CONFIG_SCHEMA)
 
+        mirror_dirs = {mirror.dir for mirror in self.mirrors}
+        if len(mirror_dirs) != len(self.mirrors):
+            raise jsonschema.ValidationError(
+                message="Multiple mirrors defined using same dir",
+                path=["mirror"],
+                instance=self.mirrors,
+            )
+
     @classmethod
     def from_file(cls, filename=".mirror-tool.yaml") -> "Config":
         with open(filename, "rt") as f:
